class BasePresenter
  attr_reader :object, :template, :model_name

  def initialize(object, template)
    @object, @template = object, template
    @model_name = @object.class.name.underscore
  end

  def method_missing(*args, &block)
    if object.respond_to? args.first
      object.send(*args, &block)
    elsif template.respond_to? args.first
      template.send(*args, &block)
    else
      super
    end
  end

  def BasePresenter.presents(model_name)
    define_method model_name do
      send(:object)
    end
  end

  def index_link(mini=false)
    link_to t('.back', :default => t("helpers.links.back")),
      template.send("#{model_name.to_s.pluralize}_path"), :class => btn_class(mini)
  end

  def edit_link(mini=false)
    link_to t('.edit', :default => t("helpers.links.edit")),
      template.send("edit_#{model_name}_path", object), :class => btn_class(mini)
  end

  def destroy_link(mini=false)
    link_to t('.destroy', :default => t("helpers.links.destroy")),
      template.send("#{model_name}_path", object),
      :method => 'delete',
      :confirm => t('.confirm', :default => t("helpers.links.confirm", :default => 'Are you sure?')),
      :class => btn_class(mini, "btn-danger")
  end

  protected
  def btn_class(mini=false, *args)
    classes = ["btn"]
    classes << "btn-mini" if mini
    classes += args
    classes.join(" ")
  end

  def handle_none(value, &blk)
    if value.blank?
      'n/a'
    else
      blk.call
    end
  end
end
